/**
 * apeman react component for modal view stack
 * @class ApModal
 */

'use strict'

import React, {PropTypes as types} from 'react'
import classnames from 'classnames'
import numcal from 'numcal'
import {ApViewStack} from 'apeman-react-view'
import {ApStyle} from 'apeman-react-style'
import {ApStackMixin} from 'apeman-react-mixin-stack'
import {ApLayoutMixin} from 'apeman-react-mixin-layout'
import uuid from 'uuid'

/** @lends ApModal */
const ApModal = React.createClass({

  // --------------------
  // Specs
  // --------------------

  propTypes: {
    /** Modal content width */
    width: types.number,
    /** Modal content height */
    height: types.number
  },

  mixins: [
    ApStackMixin,
    ApLayoutMixin
  ],

  statics: {
    VERTICAL_PADDING: 16,
    HORIZONTAL_PADDING: 32
  },

  getInitialState () {
    return {
      killScrolling: false
    }
  },

  getDefaultProps () {
    return {
      width: null,
      height: null,
      uuid: uuid.v4()
    }
  },

  render () {
    const s = this
    let { props } = s
    let { uuid } = props
    return (
      <div className={ classnames('ap-modal', `ap-modal-${uuid}`, props.className, {
        'ap-modal-active': s.hasStackedView()
      }) }
           style={ Object.assign({}, props.style) }>
        { s._renderGlobalStyle() }
        { s._renderLayoutStyle() }
        { s._renderStack() }
        { props.children }
      </div>
    )
  },

  // --------------------
  // Lifecycle
  // --------------------

  componentWillMount () {
    const s = this
  },

  componentDidMount () {
    const s = this
    s.setState({ killScrolling: s.hasStackedView() })
  },

  componentWillUnmount () {
    const s = this
  },

  // ------------------
  // Custom
  // ------------------

  hasStackedView () {
    const s = this
    let { stacker } = s.props

    return stacker && !stacker.isEmpty()
  },

  // ------------------
  // For ApLayoutMixin
  // ------------------

  getInitialLayouts () {
    const vPadding = ApModal.VERTICAL_PADDING
    const hPadding = ApModal.HORIZONTAL_PADDING;

    return {
      inner: {
        top: vPadding,
        right: hPadding,
        bottom: vPadding,
        left: hPadding,
        opacity: 0
      }
    }
  },

  calcLayouts () {
    const vPadding = ApModal.VERTICAL_PADDING
    const hPadding = ApModal.HORIZONTAL_PADDING

    const s = this
    let { props } = s
    let { outerWidth, outerHeight } = window
    let width = props.width || outerWidth
    let height = props.height || outerHeight
    let contentWidth = numcal.min(width, outerWidth, ApStyle.CONTAINER_WIDTH)
    let contentHeight = numcal.min(height, outerHeight)
    let hEdge = (outerWidth - contentWidth) / 2
    let vEdge = (outerHeight - contentHeight) / 2
    return {
      inner: {
        top: vEdge + vPadding,
        right: hEdge + hPadding,
        bottom: vEdge + vPadding,
        left: hEdge + hPadding,
        opacity: 1
      }
    }
  },

  // ------------------
  // For ApStackMixin
  // ------------------

  stackedViewDidPush (view) {
    const s = this
    s.setState({ killScrolling: s.hasStackedView() })
  },

  stackedViewDidPop (view) {
    const s = this
    s.setState({ killScrolling: s.hasStackedView() })
  },

  // ------------------
  // Private
  // ------------------

  _renderLayoutStyle () {
    const s = this
    let { layouts, props } = s

    let style = {
      [`.ap-modal-${props.uuid} .ap-view-stack-inner`]: layouts.inner
    }
    return (
      <ApStyle
        className="ap-modal-layout-style"
        data={style}>
      </ApStyle>
    )
  },

  _renderGlobalStyle () {
    const s = this
    let { state } = s

    let killScrolling = !!state.killScrolling
    let style = {
      'body,html': {
        overflow: killScrolling ? 'hidden !important' : null
      }
    }
    return (
      <ApStyle scoped={ false }
               className="ap-modal-global-style"
               data={style}>
      </ApStyle>
    )
  },

  _renderStack () {
    const s = this
    let { sate, props } = s

    let insets = s.getStackInsets()

    return (
      <ApViewStack stacker={props.stacker}
                   insetTop={insets.top || 0}
                   insetRight={insets.right || 0}
                   insetLeft={insets.left || 0}
                   insetBottom={insets.bottom || 0}
      >
      </ApViewStack>
    )
  }
})

export default ApModal
