/**
 * apeman react component for modal view stack
 * @class ApModal
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _numcal = require('numcal');

var _numcal2 = _interopRequireDefault(_numcal);

var _apemanReactView = require('apeman-react-view');

var _apemanReactStyle = require('apeman-react-style');

var _apemanReactMixinStack = require('apeman-react-mixin-stack');

var _apemanReactMixinLayout = require('apeman-react-mixin-layout');

var _uuid = require('uuid');

var _uuid2 = _interopRequireDefault(_uuid);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/** @lends ApModal */
var ApModal = _react2.default.createClass({
  displayName: 'ApModal',


  // --------------------
  // Specs
  // --------------------

  propTypes: {
    /** Modal content width */
    width: _react.PropTypes.number,
    /** Modal content height */
    height: _react.PropTypes.number
  },

  mixins: [_apemanReactMixinStack.ApStackMixin, _apemanReactMixinLayout.ApLayoutMixin],

  statics: {
    VERTICAL_PADDING: 16,
    HORIZONTAL_PADDING: 32
  },

  getInitialState: function getInitialState() {
    return {
      killScrolling: false
    };
  },
  getDefaultProps: function getDefaultProps() {
    return {
      width: null,
      height: null,
      uuid: _uuid2.default.v4()
    };
  },
  render: function render() {
    var s = this;
    var props = s.props;
    var uuid = props.uuid;

    return _react2.default.createElement(
      'div',
      { className: (0, _classnames2.default)('ap-modal', 'ap-modal-' + uuid, props.className, {
          'ap-modal-active': s.hasStackedView()
        }),
        style: Object.assign({}, props.style) },
      s._renderGlobalStyle(),
      s._renderLayoutStyle(),
      s._renderStack(),
      props.children
    );
  },


  // --------------------
  // Lifecycle
  // --------------------

  componentWillMount: function componentWillMount() {
    var s = this;
  },
  componentDidMount: function componentDidMount() {
    var s = this;
    s.setState({ killScrolling: s.hasStackedView() });
  },
  componentWillUnmount: function componentWillUnmount() {
    var s = this;
  },


  // ------------------
  // Custom
  // ------------------

  hasStackedView: function hasStackedView() {
    var s = this;
    var stacker = s.props.stacker;


    return stacker && !stacker.isEmpty();
  },


  // ------------------
  // For ApLayoutMixin
  // ------------------

  getInitialLayouts: function getInitialLayouts() {
    var vPadding = ApModal.VERTICAL_PADDING;
    var hPadding = ApModal.HORIZONTAL_PADDING;

    return {
      inner: {
        top: vPadding,
        right: hPadding,
        bottom: vPadding,
        left: hPadding,
        opacity: 0
      }
    };
  },
  calcLayouts: function calcLayouts() {
    var vPadding = ApModal.VERTICAL_PADDING;
    var hPadding = ApModal.HORIZONTAL_PADDING;

    var s = this;
    var props = s.props;
    var _window = window;
    var outerWidth = _window.outerWidth;
    var outerHeight = _window.outerHeight;

    var width = props.width || outerWidth;
    var height = props.height || outerHeight;
    var contentWidth = _numcal2.default.min(width, outerWidth, _apemanReactStyle.ApStyle.CONTAINER_WIDTH);
    var contentHeight = _numcal2.default.min(height, outerHeight);
    var hEdge = (outerWidth - contentWidth) / 2;
    var vEdge = (outerHeight - contentHeight) / 2;
    return {
      inner: {
        top: vEdge + vPadding,
        right: hEdge + hPadding,
        bottom: vEdge + vPadding,
        left: hEdge + hPadding,
        opacity: 1
      }
    };
  },


  // ------------------
  // For ApStackMixin
  // ------------------

  stackedViewDidPush: function stackedViewDidPush(view) {
    var s = this;
    s.setState({ killScrolling: s.hasStackedView() });
  },
  stackedViewDidPop: function stackedViewDidPop(view) {
    var s = this;
    s.setState({ killScrolling: s.hasStackedView() });
  },


  // ------------------
  // Private
  // ------------------

  _renderLayoutStyle: function _renderLayoutStyle() {
    var s = this;
    var layouts = s.layouts;
    var props = s.props;


    var style = _defineProperty({}, '.ap-modal-' + props.uuid + ' .ap-view-stack-inner', layouts.inner);
    return _react2.default.createElement(_apemanReactStyle.ApStyle, {
      className: 'ap-modal-layout-style',
      data: style });
  },
  _renderGlobalStyle: function _renderGlobalStyle() {
    var s = this;
    var state = s.state;


    var killScrolling = !!state.killScrolling;
    var style = {
      'body,html': {
        overflow: killScrolling ? 'hidden !important' : null
      }
    };
    return _react2.default.createElement(_apemanReactStyle.ApStyle, { scoped: false,
      className: 'ap-modal-global-style',
      data: style });
  },
  _renderStack: function _renderStack() {
    var s = this;
    var sate = s.sate;
    var props = s.props;


    var insets = s.getStackInsets();

    return _react2.default.createElement(_apemanReactView.ApViewStack, { stacker: props.stacker,
      insetTop: insets.top || 0,
      insetRight: insets.right || 0,
      insetLeft: insets.left || 0,
      insetBottom: insets.bottom || 0
    });
  }
});

exports.default = ApModal;
//# sourceMappingURL=data:application/json;base64,bnVsbA==